@@ -1,6 +1,8 @@
-require 'action_mailer/vendor/text/format'
+require 'text/format'
 
-module MailHelper#:nodoc:
+module MailHelper
+  # Uses Text::Format to take the text and format it, indented two spaces for
+  # each line, and wrapped at 72 columns.
   def block_format(text)
     formatted = text.split(/\n\r\n/).collect { |paragraph| 
       Text::Format.new(
@@ -14,4 +16,4 @@ module MailHelper#:nodoc:
 
     formatted
   end
-end
\ No newline at end of file
+end
