@@ -1,29 +1,9 @@
-$:.unshift(File.dirname(__FILE__) + "/../lib/")
-
-require 'test/unit'
-require 'action_mailer'
-
-class MockSMTP
-  def self.deliveries
-    @@deliveries
-  end
-
-  def initialize
-    @@deliveries = []
-  end
-
-  def sendmail(mail, from, to)
-    @@deliveries << [mail, from, to]
-  end
-end
-
-class Net::SMTP
-  def self.start(*args)
-    yield MockSMTP.new
-  end
-end
+# encoding: utf-8
+require 'abstract_unit'
 
 class FunkyPathMailer < ActionMailer::Base
+  self.template_root = "#{File.dirname(__FILE__)}/fixtures/path.with.dots"
+
   def multipart_with_template_path_with_dots(recipient)
     recipients recipient
     subject    "Have a lovely picture"
@@ -31,14 +11,9 @@ class FunkyPathMailer < ActionMailer::Base
     attachment :content_type => "image/jpeg",
       :body => "not really a jpeg, we're only testing, after all"
   end
-
-  def template_path
-    "#{File.dirname(__FILE__)}/fixtures/path.with.dots"
-  end
 end
 
 class TestMailer < ActionMailer::Base
-
   def signed_up(recipient)
     @recipients   = recipient
     @subject      = "[Signed up] Welcome #{recipient}"
@@ -65,6 +40,15 @@ class TestMailer < ActionMailer::Base
     body       "Nothing to see here."
   end
 
+  def different_reply_to(recipient)
+    recipients recipient
+    subject    "testing reply_to"
+    from       "system@loudthinking.com"
+    sent_on    Time.local(2008, 5, 23)
+    reply_to   "atraver@gmail.com"
+    body       "Nothing to see here."
+  end
+
   def iso_charset(recipient)
     @recipients = recipient
     @subject    = "testing isø charsets"
@@ -198,6 +182,15 @@ class TestMailer < ActionMailer::Base
     body["recipient"] = recipient
   end
 
+  def custom_templating_extension(recipient)
+    recipients recipient
+    subject    "[Signed up] Welcome #{recipient}"
+    from       "system@loudthinking.com"
+    sent_on    Time.local(2004, 12, 12)
+
+    body["recipient"] = recipient
+  end
+
   def various_newlines(recipient)
     recipients   recipient
     subject      "various newlines"
@@ -220,13 +213,23 @@ class TestMailer < ActionMailer::Base
     subject      "nested multipart"
     from         "test@example.com"
     content_type "multipart/mixed"
-    part :content_type => "multipart/alternative", :content_disposition => "inline" do |p|
+    part :content_type => "multipart/alternative", :content_disposition => "inline", :headers => { "foo" => "bar" } do |p|
       p.part :content_type => "text/plain", :body => "test text\nline #2"
       p.part :content_type => "text/html", :body => "<b>test</b> HTML<br/>\nline #2"
     end
     attachment :content_type => "application/octet-stream",:filename => "test.txt", :body => "test abcdefghijklmnopqstuvwxyz"
   end
   
+  def nested_multipart_with_body(recipient)
+    recipients   recipient
+    subject      "nested multipart with body"
+    from         "test@example.com"
+    content_type "multipart/mixed"
+    part :content_type => "multipart/alternative", :content_disposition => "inline", :body => "Nothing to see here." do |p|
+      p.part :content_type => "text/html", :body => "<b>test</b> HTML<br/>"
+    end
+  end
+
   def attachment_with_custom_header(recipient)
     recipients   recipient
     subject      "custom header in attachment"
@@ -262,6 +265,14 @@ class TestMailer < ActionMailer::Base
     body         "testing"
   end
 
+  def return_path
+    recipients   "no.one@nowhere.test"
+    subject      "return path test"
+    from         "some.one@somewhere.test"
+    body         "testing"
+    headers      "return-path" => "another@somewhere.test"
+  end
+
   class <<self
     attr_accessor :received_body
   end
@@ -271,7 +282,7 @@ class TestMailer < ActionMailer::Base
   end
 end
 
-TestMailer.template_root = File.dirname(__FILE__) + "/fixtures"
+uses_mocha 'ActionMailerTest' do
 
 class ActionMailerTest < Test::Unit::TestCase
   include ActionMailer::Quoting
@@ -282,20 +293,29 @@ class ActionMailerTest < Test::Unit::TestCase
 
   def new_mail( charset="utf-8" )
     mail = TMail::Mail.new
+    mail.mime_version = "1.0"
     if charset
       mail.set_content_type "text", "plain", { "charset" => charset }
     end
     mail
   end
 
+  # Replacing logger work around for mocha bug. Should be fixed in mocha 0.3.3
   def setup
-    ActionMailer::Base.delivery_method = :test
+    set_delivery_method :test
     ActionMailer::Base.perform_deliveries = true
+    ActionMailer::Base.raise_delivery_errors = true
     ActionMailer::Base.deliveries = []
 
+    @original_logger = TestMailer.logger
     @recipient = 'test@localhost'
   end
 
+  def teardown
+    TestMailer.logger = @original_logger
+    restore_delivery_method
+  end
+
   def test_nested_parts
     created = nil
     assert_nothing_raised { created = TestMailer.create_nested_multipart(@recipient)}
@@ -304,11 +324,25 @@ class ActionMailerTest < Test::Unit::TestCase
     
     assert_equal "multipart/mixed", created.content_type
     assert_equal "multipart/alternative", created.parts.first.content_type
+    assert_equal "bar", created.parts.first.header['foo'].to_s
     assert_equal "text/plain", created.parts.first.parts.first.content_type
     assert_equal "text/html", created.parts.first.parts[1].content_type
     assert_equal "application/octet-stream", created.parts[1].content_type
   end
 
+  def test_nested_parts_with_body
+    created = nil
+    assert_nothing_raised { created = TestMailer.create_nested_multipart_with_body(@recipient)}
+    assert_equal 1,created.parts.size
+    assert_equal 2,created.parts.first.parts.size
+
+    assert_equal "multipart/mixed", created.content_type
+    assert_equal "multipart/alternative", created.parts.first.content_type
+    assert_equal "Nothing to see here.", created.parts.first.parts.first.body
+    assert_equal "text/plain", created.parts.first.parts.first.content_type
+    assert_equal "text/html", created.parts.first.parts[1].content_type
+  end
+
   def test_attachment_with_custom_header
     created = nil
     assert_nothing_raised { created = TestMailer.create_attachment_with_custom_header(@recipient)}
@@ -322,7 +356,6 @@ class ActionMailerTest < Test::Unit::TestCase
     expected.body    = "Hello there, \n\nMr. #{@recipient}"
     expected.from    = "system@loudthinking.com"
     expected.date    = Time.local(2004, 12, 12)
-    expected.mime_version = nil
 
     created = nil
     assert_nothing_raised { created = TestMailer.create_signed_up(@recipient) }
@@ -347,7 +380,37 @@ class ActionMailerTest < Test::Unit::TestCase
     assert_not_nil created
     assert_equal expected.encoded, created.encoded
   end
-  
+
+  def test_custom_templating_extension
+    #
+    # N.b., custom_templating_extension.text.plain.haml is expected to be in fixtures/test_mailer directory
+    expected = new_mail
+    expected.to      = @recipient
+    expected.subject = "[Signed up] Welcome #{@recipient}"
+    expected.body    = "Hello there, \n\nMr. #{@recipient}"
+    expected.from    = "system@loudthinking.com"
+    expected.date    = Time.local(2004, 12, 12)
+    
+    # Stub the render method so no alternative renderers need be present.
+    ActionView::Base.any_instance.stubs(:render).returns("Hello there, \n\nMr. #{@recipient}")
+    
+    # If the template is not registered, there should be no parts.
+    created = nil
+    assert_nothing_raised { created = TestMailer.create_custom_templating_extension(@recipient) }
+    assert_not_nil created
+    assert_equal 0, created.parts.length
+    
+    ActionMailer::Base.register_template_extension('haml')
+    
+    # Now that the template is registered, there should be one part. The text/plain part.
+    created = nil
+    assert_nothing_raised { created = TestMailer.create_custom_templating_extension(@recipient) }
+    assert_not_nil created
+    assert_equal 2, created.parts.length
+    assert_equal 'text/plain', created.parts[0].content_type
+    assert_equal 'text/html', created.parts[1].content_type
+  end
+
   def test_cancelled_account
     expected = new_mail
     expected.to      = @recipient
@@ -391,6 +454,31 @@ class ActionMailerTest < Test::Unit::TestCase
     assert_equal expected.encoded, ActionMailer::Base.deliveries.first.encoded
   end
 
+  def test_reply_to
+    expected = new_mail
+
+    expected.to       = @recipient
+    expected.subject  = "testing reply_to"
+    expected.body     = "Nothing to see here."
+    expected.from     = "system@loudthinking.com"
+    expected.reply_to = "atraver@gmail.com"
+    expected.date     = Time.local 2008, 5, 23
+
+    created = nil
+    assert_nothing_raised do
+      created = TestMailer.create_different_reply_to @recipient
+    end
+    assert_not_nil created
+    assert_equal expected.encoded, created.encoded
+
+    assert_nothing_raised do
+      TestMailer.deliver_different_reply_to @recipient
+    end
+
+    assert_not_nil ActionMailer::Base.deliveries.first
+    assert_equal expected.encoded, ActionMailer::Base.deliveries.first.encoded
+  end
+
   def test_iso_charset
     expected = new_mail( "iso-8859-1" )
     expected.to      = @recipient
@@ -462,6 +550,30 @@ class ActionMailerTest < Test::Unit::TestCase
     TestMailer.deliver_signed_up(@recipient)
     assert_equal 1, ActionMailer::Base.deliveries.size
   end
+  
+  def test_doesnt_raise_errors_when_raise_delivery_errors_is_false
+    ActionMailer::Base.raise_delivery_errors = false
+    TestMailer.any_instance.expects(:perform_delivery_test).raises(Exception)
+    assert_nothing_raised { TestMailer.deliver_signed_up(@recipient) }
+  end
+
+  def test_performs_delivery_via_sendmail
+    sm = mock()
+    sm.expects(:print).with(anything)
+    sm.expects(:flush)
+    IO.expects(:popen).once.with('/usr/sbin/sendmail -i -t', 'w+').yields(sm)
+    ActionMailer::Base.delivery_method = :sendmail
+    TestMailer.deliver_signed_up(@recipient)
+  end
+
+  def test_delivery_logs_sent_mail
+    mail = TestMailer.create_signed_up(@recipient)
+    logger = mock()
+    logger.expects(:info).with("Sent mail to #{@recipient}")
+    logger.expects(:debug).with("\n#{mail.encoded}")
+    TestMailer.logger = logger
+    TestMailer.deliver_signed_up(@recipient)
+  end
 
   def test_unquote_quoted_printable_subject
     msg = <<EOF
@@ -762,7 +874,11 @@ EOF
     fixture = File.read(File.dirname(__FILE__) + "/fixtures/raw_email8")
     mail = TMail::Mail.parse(fixture)
     attachment = mail.attachments.last
-    assert_equal "01QuienTeDijat.Pitbull.mp3", attachment.original_filename
+
+    expected = "01 Quien Te Dij\212at. Pitbull.mp3"
+    expected.force_encoding(Encoding::ASCII_8BIT) if expected.respond_to?(:force_encoding)
+
+    assert_equal expected, attachment.original_filename
   end
 
   def test_wrong_mail_header
@@ -776,12 +892,6 @@ EOF
     assert_nothing_raised { mail.body }
   end
 
-  def test_decode_message_with_unquoted_atchar_in_header
-    fixture = File.read(File.dirname(__FILE__) + "/fixtures/raw_email11")
-    mail = TMail::Mail.parse(fixture)
-    assert_not_nil mail.from
-  end
-
   def test_empty_header_values_omitted
     result = TestMailer.create_unnamed_attachment(@recipient).encoded
     assert_match %r{Content-Type: application/octet-stream[^;]}, result
@@ -814,5 +924,56 @@ EOF
     assert_match %r{format=flowed}, mail['content-type'].to_s
     assert_match %r{charset=utf-8}, mail['content-type'].to_s
   end
+
+  def test_return_path_with_create
+    mail = TestMailer.create_return_path
+    assert_equal "<another@somewhere.test>", mail['return-path'].to_s
+  end
+
+  def test_return_path_with_deliver
+    ActionMailer::Base.delivery_method = :smtp
+    TestMailer.deliver_return_path
+    assert_match %r{^Return-Path: <another@somewhere.test>}, MockSMTP.deliveries[0][0]
+  end
 end
 
+end # uses_mocha
+
+class InheritableTemplateRootTest < Test::Unit::TestCase
+  def test_attr
+    expected = "#{File.dirname(__FILE__)}/fixtures/path.with.dots"
+    assert_equal expected, FunkyPathMailer.template_root
+
+    sub = Class.new(FunkyPathMailer)
+    sub.template_root = 'test/path'
+
+    assert_equal 'test/path', sub.template_root
+    assert_equal expected, FunkyPathMailer.template_root
+  end
+end
+
+class MethodNamingTest < Test::Unit::TestCase
+  class TestMailer < ActionMailer::Base
+    def send
+      body 'foo'
+    end
+  end
+
+  def setup
+    set_delivery_method :test
+    ActionMailer::Base.perform_deliveries = true
+    ActionMailer::Base.deliveries = []
+  end
+
+  def teardown
+    restore_delivery_method
+  end
+
+  def test_send_method
+    assert_nothing_raised do
+      assert_emails 1 do
+        TestMailer.deliver_send
+      end
+    end
+  end
+end
