@@ -1,10 +1,10 @@
-#!/usr/local/bin/ruby
+#!/usr/bin/env ruby
 
 VERSION  = ARGV.first
-PACKAGES = %w( activesupport activerecord actionpack actionmailer actionwebservice )
+PACKAGES = %w(activesupport activerecord actionpack actionmailer activeresource)
 
 # Checkout source
-`rm -rf release && svn export http://dev.rubyonrails.org/svn/rails/trunk release`
+# `rm -rf release && svn export http://dev.rubyonrails.org/svn/rails/trunk release`
 
 # Create Rails packages
 `cd release/railties && rake template=jamis package`
@@ -18,8 +18,5 @@ PACKAGES = %w( activesupport activerecord actionpack actionmailer actionwebservi
 end
 
 # Upload rails tgz/zip
-`rubyforge add_release rails rails 'REL #{VERSION}' rails-#{VERSION}.tgz`
-`rubyforge add_release rails rails 'REL #{VERSION}' rails-#{VERSION}.zip`
-
-# Create SVN tag
-puts "Remeber to create SVN tag"
\ No newline at end of file
+`rubyforge add_release rails rails 'REL #{VERSION}' release/rails-#{VERSION}.tgz`
+`rubyforge add_release rails rails 'REL #{VERSION}' release/rails-#{VERSION}.zip`
\ No newline at end of file
